@@ -1,4 +1,7 @@
-# Dist/Zilla/Role/ErrorLogger.pm #
+#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+#
+#   file: Dist/Zilla/Role/ErrorLogger.pm
+#
 
 #pod =encoding UTF-8
 #pod
@@ -21,8 +24,12 @@
 #pod
 #pod =cut
 
-# --------------------------------------------------------------------------------------------------
+#   - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
+#pod =for :this This is C<Dist::Zilla::Role::ErrorLogger> role documentation. Read this if you want to
+#pod have error logging capabilities in your Dist::Zilla plugin.
+#pod
+#pod
 #pod =head1 SYNOPSIS
 #pod
 #pod     package Dist::Zilla::Plugin::YourPlugin;
@@ -35,15 +42,13 @@
 #pod         my $self = shift( @_ );
 #pod         …
 #pod         if ( … ) { $self->log_error( 'error message' ); };
-#pod         assertion_condition or $self->log_error( 'another error message' );
+#pod         do_something or $self->log_error( 'another error message' );
 #pod         …
-#pod         foreach … {
-#pod             …
-#pod             check or $self->log_error( 'error message' ) and next;
-#pod             …
+#pod         while ( … ) {
+#pod             do_something_else or $self->log_error( 'error message' ) and next;
 #pod         };
 #pod         …
-#pod         $self->abort_if_error();
+#pod         $self->abort_if_error( 'errors found' );
 #pod     };
 #pod
 #pod     __PACKAGE__->meta->make_immutable;
@@ -55,7 +60,192 @@
 
 #pod =head1 DESCRIPTION
 #pod
-#pod C<Dist::Zilla> limits logging capabilities with 3 logging levels available in plugings through
+#pod TODO
+#pod
+#pod The role requires C<log> and C<log_fatal> methods in the consumer.
+#pod
+#pod =cut
+
+# --------------------------------------------------------------------------------------------------
+
+package Dist::Zilla::Role::ErrorLogger;
+
+use Moose::Role;
+use feature qw{ state };
+use namespace::autoclean;
+
+# ABSTRACT: Have error logging capabilities in your Dist::Zilla plugin
+our $VERSION = '0.004'; # VERSION
+
+requires qw{ log log_fatal };
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =attr error_count
+#pod
+#pod     $int = $self->error_count;
+#pod
+#pod Int, read-only. Number of logged errors (i. e. number of made C<log_error> calls).
+#pod
+#pod =cut
+
+has error_count => (
+    is          => 'ro',
+    isa         => 'Int',
+    default     => 0,
+    init_arg    => undef,
+);
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =method log_error
+#pod
+#pod     $self->log_error( @items );
+#pod     $self->log_error( \%args, @items );
+#pod
+#pod This method calls C<log> method, passing all the arguments, and increments value of C<error_count>
+#pod attribute. The method returns true value, so can be used in following constructs:
+#pod
+#pod     while ( … ) {
+#pod         do_something or $self->log_error( 'message' ) and next;
+#pod     };
+#pod
+#pod =cut
+
+sub log_error {
+    my $self = shift( @_ );
+    #   If the first argument is a hashref, it is treated as extra arguments to logging, not as
+    #   message, see <https://metacpan.org/pod/Log::Dispatchouli#log>. Let us add `level` argument.
+    #   However, we should not change user-supplied hash and arguments.
+    #   ATTENTION:
+    #       If we specify level, the test passes but messages do not appear in `dzil` output. So,
+    #       level is (temporary?) commented out.
+    #   TODO: Investigate the problem.
+    state $level = {
+        #~ level => 'error',
+    };
+    my $arg;
+    if ( ref( $_[ 0 ] ) eq 'HASH' ) {
+        #   User provided hashref, remove it from `@_`.
+        $arg = shift( @_ );
+        #   Do not override user-supplied `level` argument, if any.
+        if ( not exists( $arg->{ level } ) ) {
+            #   User did not provide `level` argument, so we need to add it.
+            #   Note: new hash created intentionally, to keep user-supplied hash intact.
+            $arg = { %$level, %$arg };
+        };
+    } else {
+        #   User did not provide hashref, just use ours.
+        $arg = $level;
+    };
+    $self->log( $arg, @_ );
+    ++ $self->{ error_count };
+    return $self->{ error_count };
+};
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =method abort
+#pod
+#pod     $self->abort( @items );
+#pod     $self->abort( \%args, @items );
+#pod
+#pod This method logs all the arguments and aborts C<Dist::Zilla>. Does it look similar to C<log_fatal>?
+#pod Yep. This is attempt to workaround L<C<log_fatal>
+#pod drawback|https://github.com/rjbs/Dist-Zilla/issues/397>. In contrast to C<log_fatal>, C<abort>
+#pod guarantees the message appears on the screen only once.
+#pod
+#pod TODO: Describe exception.
+#pod
+#pod =cut
+
+sub abort {
+    my $self = shift( @_ );
+    if ( @_ ) {
+        $self->log( @_ );
+    };
+    #~ $self->log_fatal( "Aborting..." );
+    die( "Aborting...\n" );
+};
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =method abort_if_error
+#pod
+#pod     $self->abort_if_error( @items );
+#pod     $self->abort_if_error( \%args, @items );
+#pod
+#pod If there was any errors (i. e. C<error_count> is greater than zero), the logs all the arguments and
+#pod aborts execution. Both actions (logging and aborting) are implemented by calling C<abort>.
+#pod
+#pod =cut
+
+sub abort_if_error {
+    my $self = shift( @_ );
+    if ( $self->error_count ) {
+        $self->abort( @_  );
+    };
+};
+
+# --------------------------------------------------------------------------------------------------
+
+1;
+
+# --------------------------------------------------------------------------------------------------
+
+#pod =head1 NOTES
+#pod
+#pod All the methods defined in the role log items through the C<log> method. C<Dist::Zilla> takes this
+#pod method from C<Log::Dispatchouli>, the latter uses C<String::Flogger> to process the messages. It
+#pod means you can use C<String::Flogger> tricks, e. g.:
+#pod
+#pod     $self->log_error( [ 'oops at %s line %d', $file, $line ] );
+#pod         #   [] are shorter than sprintf.
+#pod
+#pod Also note how C<Log::Dispatchouli> describes the C<log> method:
+#pod
+#pod     $logger->log( @messages );
+#pod
+#pod and says:
+#pod
+#pod     Each message is flogged individually, then joined with spaces.
+#pod
+#pod So beware. A call
+#pod
+#pod     $self->log_error( 'error 1', 'error 2' );
+#pod
+#pod logs I<one message> "error 1 error 2", I<not> two messages "error 1" and "error 2", and bumps
+#pod C<error_count> by 1, not 2.
+#pod
+#pod =head1 SEE ALSO
+#pod
+#pod L<Dist::Zilla>
+#pod L<Dist::Zilla::Role>
+#pod L<Dist::Zilla::Plugin>
+#pod L<Log::Dispatchouli>
+#pod L<String::Flogger>
+#pod
+#pod =cut
+
+# doc/what.pod #
+
+#pod =encoding UTF-8
+#pod
+#pod =head1 WHAT?
+#pod
+#pod C<Dist-Zilla-Role-ErrorLogger> is a C<Dist::Zilla> role. It provides C<log_error>, C<abort>, and
+#pod C<abort_if_error> methods to consuming plugin.
+#pod
+#pod =cut
+
+# end of file #
+# doc/why.pod #
+
+#pod =encoding UTF-8
+#pod
+#pod =head1 WHY?
+#pod
+#pod C<Dist::Zilla> limits logging capabilities with 3 logging levels available in plugins through
 #pod C<log_debug>, C<log>, and C<log_fatal> methods. Debug level messages are turned off by default, the
 #pod first fatal message terminates C<Dist::Zilla>. This is simple, but sometimes you may want to report
 #pod all the errors, instead of stopping at the first found one. In such a case C<log_fatal> cannot be
@@ -90,7 +280,7 @@
 #pod This works, but incrementing the counter after each C<log> call is boring and error-prone.
 #pod C<Dist-Zilla-Role-ErrorLogger> role automates it, making plugin code shorter and more readable:
 #pod
-#pod     with 'Dist::Zilla::Role::ErrorLogger';
+#pod     with 'Dist-Zilla-Role-ErrorLogger';
 #pod     …
 #pod     $self->log_error( 'error' );
 #pod     …
@@ -98,107 +288,8 @@
 #pod
 #pod =cut
 
-# --------------------------------------------------------------------------------------------------
+# end of file #
 
-package Dist::Zilla::Role::ErrorLogger;
-
-use feature qw{ state };
-
-# ABSTRACT: Provide error logging capabilities for Dist::Zilla plugins
-
-BEGIN {
-    our $VERSION = '0.003'; # VERSION
-};
-
-use Moose::Role;
-use namespace::autoclean;
-
-requires qw{ log log_fatal };
-
-# --------------------------------------------------------------------------------------------------
-
-#pod =attr error_count
-#pod
-#pod Number of logged errors (i. e. number of C<log_error> calls). Read-only.
-#pod
-#pod =cut
-
-has error_count => (
-    is          => 'ro',
-    isa         => 'Int',
-    default     => 0,
-    init_arg    => undef,
-);
-
-# --------------------------------------------------------------------------------------------------
-
-#pod =method log_error
-#pod
-#pod This method calls C<log> method, passing all the arguments, and increments value of C<error_count>
-#pod attribute. The method returns true value, so can be used in following constructs:
-#pod
-#pod     while ( … ) {
-#pod         …
-#pod         assert_condition or $self->log_error( 'message' ) and next;
-#pod         …
-#pod     };
-#pod
-#pod Messages are reported at C<error> level, if level is not explicitly specified. (It seems level is
-#pod not used by C<Dist::Zilla> now.) Custom arguments are respected, too:
-#pod
-#pod     $self->log_error( { prefix => 'subsystem: ', level => 'info' }, $message );
-#pod
-#pod (See L<Log::Dispatchouli/log> for more details. Err… Unfortunately, there are very little details
-#pod there.)
-#pod
-#pod =cut
-
-sub log_error {
-    my $self = shift( @_ );
-    #   If the first argument is a hashref, it is treated as extra arguments to logging, not as
-    #   message, see <https://metacpan.org/pod/Log::Dispatchouli#log>. Let us add `level` argument.
-    #   However, we should not change user-supplied hash and arguments.
-    state $level = {
-        level => 'error',
-    };
-    my $arg;
-    if ( ref( $_[ 0 ] ) eq 'HASH' ) {
-        #   User provided hashref, remove it from `@_`.
-        $arg = shift( @_ );
-        #   Do not override user-supplied `level` argument, if any.
-        if ( not exists( $arg->{ level } ) ) {
-            #   User did not provide `level` argument, so we need to add it.
-            #   Note: new hash created intentionally, to keep user-supplied hash intact.
-            $arg = { %$level, %$arg };
-        };
-    } else {
-        #   User did not provide hashref, just use ours.
-        $arg = $level;
-    };
-    $self->log( $arg, @_ );
-    ++ $self->{ error_count };
-    return $self->{ error_count };
-};
-
-# --------------------------------------------------------------------------------------------------
-
-#pod =method abort_if_error
-#pod
-#pod If there was any errors (i. e. C<error_count> is greater than zero), the method aborts execution by
-#pod calling C<log_fatal> with all the arguments. If the method is called with no arguments, it pass
-#pod to C<logh_fatal> string "Aborting...".
-#pod
-#pod =cut
-
-sub abort_if_error {
-    my $self = shift( @_ );
-    if ( $self->error_count ) {
-        $self->log_fatal( @_ ? @_ : 'Aborting...' );
-            #   `@_ or 'Aborting...'` looks shorter, but it evaluates `@_` in scalar context.
-    };
-};
-
-1;
 
 # end of file #
 
@@ -210,11 +301,19 @@ __END__
 
 =head1 NAME
 
-Dist::Zilla::Role::ErrorLogger - Provide error logging capabilities for Dist::Zilla plugins
+Dist::Zilla::Role::ErrorLogger - Have error logging capabilities in your Dist::Zilla plugin
 
 =head1 VERSION
 
-Version 0.003.
+Version 0.004, released on 2015-07-14 18:48 UTC.
+
+=head1 WHAT?
+
+C<Dist-Zilla-Role-ErrorLogger> is a C<Dist::Zilla> role. It provides C<log_error>, C<abort>, and
+C<abort_if_error> methods to consuming plugin.
+
+This is C<Dist::Zilla::Role::ErrorLogger> role documentation. Read this if you want to
+have error logging capabilities in your Dist::Zilla plugin.
 
 =head1 SYNOPSIS
 
@@ -228,15 +327,13 @@ Version 0.003.
         my $self = shift( @_ );
         …
         if ( … ) { $self->log_error( 'error message' ); };
-        assertion_condition or $self->log_error( 'another error message' );
+        do_something or $self->log_error( 'another error message' );
         …
-        foreach … {
-            …
-            check or $self->log_error( 'error message' ) and next;
-            …
+        while ( … ) {
+            do_something_else or $self->log_error( 'error message' ) and next;
         };
         …
-        $self->abort_if_error();
+        $self->abort_if_error( 'errors found' );
     };
 
     __PACKAGE__->meta->make_immutable;
@@ -244,7 +341,79 @@ Version 0.003.
 
 =head1 DESCRIPTION
 
-C<Dist::Zilla> limits logging capabilities with 3 logging levels available in plugings through
+TODO
+
+The role requires C<log> and C<log_fatal> methods in the consumer.
+
+=head1 OBJECT ATTRIBUTES
+
+=head2 error_count
+
+    $int = $self->error_count;
+
+Int, read-only. Number of logged errors (i. e. number of made C<log_error> calls).
+
+=head1 OBJECT METHODS
+
+=head2 log_error
+
+    $self->log_error( @items );
+    $self->log_error( \%args, @items );
+
+This method calls C<log> method, passing all the arguments, and increments value of C<error_count>
+attribute. The method returns true value, so can be used in following constructs:
+
+    while ( … ) {
+        do_something or $self->log_error( 'message' ) and next;
+    };
+
+=head2 abort
+
+    $self->abort( @items );
+    $self->abort( \%args, @items );
+
+This method logs all the arguments and aborts C<Dist::Zilla>. Does it look similar to C<log_fatal>?
+Yep. This is attempt to workaround L<C<log_fatal>
+drawback|https://github.com/rjbs/Dist-Zilla/issues/397>. In contrast to C<log_fatal>, C<abort>
+guarantees the message appears on the screen only once.
+
+TODO: Describe exception.
+
+=head2 abort_if_error
+
+    $self->abort_if_error( @items );
+    $self->abort_if_error( \%args, @items );
+
+If there was any errors (i. e. C<error_count> is greater than zero), the logs all the arguments and
+aborts execution. Both actions (logging and aborting) are implemented by calling C<abort>.
+
+=head1 NOTES
+
+All the methods defined in the role log items through the C<log> method. C<Dist::Zilla> takes this
+method from C<Log::Dispatchouli>, the latter uses C<String::Flogger> to process the messages. It
+means you can use C<String::Flogger> tricks, e. g.:
+
+    $self->log_error( [ 'oops at %s line %d', $file, $line ] );
+        #   [] are shorter than sprintf.
+
+Also note how C<Log::Dispatchouli> describes the C<log> method:
+
+    $logger->log( @messages );
+
+and says:
+
+    Each message is flogged individually, then joined with spaces.
+
+So beware. A call
+
+    $self->log_error( 'error 1', 'error 2' );
+
+logs I<one message> "error 1 error 2", I<not> two messages "error 1" and "error 2", and bumps
+C<error_count> by 1, not 2.
+
+=head1 WHY?
+
+C<Dist::Zilla> limits logging capabilities with 3 logging levels available in plugins through
 C<log_debug>, C<log>, and C<log_fatal> methods. Debug level messages are turned off by default, the
 first fatal message terminates C<Dist::Zilla>. This is simple, but sometimes you may want to report
 all the errors, instead of stopping at the first found one. In such a case C<log_fatal> cannot be
@@ -279,44 +448,19 @@ errors, and calling C<log_fatal> once at the end:
 This works, but incrementing the counter after each C<log> call is boring and error-prone.
 C<Dist-Zilla-Role-ErrorLogger> role automates it, making plugin code shorter and more readable:
 
-    with 'Dist::Zilla::Role::ErrorLogger';
+    with 'Dist-Zilla-Role-ErrorLogger';
     …
     $self->log_error( 'error' );
     …
     $self->abort_if_error();
 
-=head1 ATTRIBUTES
+=head1 SEE ALSO
 
-=head2 error_count
-
-Number of logged errors (i. e. number of C<log_error> calls). Read-only.
-
-=head1 METHODS
-
-=head2 log_error
-
-This method calls C<log> method, passing all the arguments, and increments value of C<error_count>
-attribute. The method returns true value, so can be used in following constructs:
-
-    while ( … ) {
-        …
-        assert_condition or $self->log_error( 'message' ) and next;
-        …
-    };
-
-Messages are reported at C<error> level, if level is not explicitly specified. (It seems level is
-not used by C<Dist::Zilla> now.) Custom arguments are respected, too:
-
-    $self->log_error( { prefix => 'subsystem: ', level => 'info' }, $message );
-
-(See L<Log::Dispatchouli/log> for more details. Err… Unfortunately, there are very little details
-there.)
-
-=head2 abort_if_error
-
-If there was any errors (i. e. C<error_count> is greater than zero), the method aborts execution by
-calling C<log_fatal> with all the arguments. If the method is called with no arguments, it pass
-to C<logh_fatal> string "Aborting...".
+L<Dist::Zilla>
+L<Dist::Zilla::Role>
+L<Dist::Zilla::Plugin>
+L<Log::Dispatchouli>
+L<String::Flogger>
 
 =head1 AUTHOR
 
